# interactive.nu -- 交互模块
#
# Copyright (C) 2022 KAAAsS

def --env confirm_loop [ prompt: string, default: string ] {
    $env.result = if ($env.result == null) {
        let got = (input $prompt)
        if ($got == "y" or $got == "n") {
            $got
        } else if ($got == "" and $default != null) {
            $default
        } else {
            null
        }
    } else {
        $env.result
    }
}

# 询问用户是否问题
export def confirm [
    prompt: string,    # 提示信息
    default?: string,  # 默认值，y 或 n
] {
    use log.nu

    # 提示符
    let indicator = if ($default == "y") {
        "Y/n"
    } else if ($default == "n") {
        "y/N"
    } else {
        "y/n"
    }
    let prompt = $"($prompt) [($indicator)] "

    if ('NO_CONFIRM' in $env and $env.NO_CONFIRM) {
        print $"($prompt)($default)"
        if ($default == null) {
            log error $"必须手动确认！"
            exit 1
        }
        $default == "y"
    } else {
        # 尝试获得结果
        $env.result = null

        # 最多尝试 5 次（因为 nushell 的限制暂时无法在循环里编辑环境）
        confirm_loop $prompt $default
        confirm_loop $prompt $default
        confirm_loop $prompt $default
        confirm_loop $prompt $default
        confirm_loop $prompt $default

        # 超过次数仍未成功
        if ($env.result == null) {
            log error "超过最大尝试次数！"
            exit 1
        }

        $env.result == "y"
    }
}

def main [] {
    print "测试 confirm"
    print (confirm "测试 1")
    print (confirm "测试 2" "y")
    print (confirm "测试 3" "n")
    $env.NO_CONFIRM = true
    print (confirm "测试 2" "y")
    print (confirm "测试 3" "n")
    print (confirm "测试 1")
}
